/*
 * file:    Counter.c
 * purpose: functions for counting of frequencies
 */
#include <inttypes.h>
#include <avr/io.h>
#include "Config.h"
#include "Data.h"
#include "Util.h"
#include "SegmentLed.h"


/*
 * initialize and start the frequency counter
 */
void initCounter(void) {
   work.counter.value = 0;
   listen2User(1);                                         // frequency counter needs 8bit timer too 
   work.stopAt = work.timer + TPS;                         // count frequency for one second
   TCCR1B = 1 << CS12 | 1 << CS11 | 1 << CS10;             // T1 is clock source for 16bit timer (frequency counting)
}


/*
 * convert a number to 5 digit display
 * first digit is reserved for unit
 */
void cnv2Display(uint32_t v) {
   uint8_t  buf[12] = {0};
    
   l2a(v, buf);
   if (v > 1000000) {
      work.display[4] = SEG_MEGA;

      work.display[3] = a2led(buf[0]) | DOT_SEG;           // we can't count more than 8 MHz, so first digit will always be dotted
      work.display[2] = a2led(buf[1]);
      work.display[1] = a2led(buf[2]);
      work.display[0] = a2led(buf[3]);
   } else if (v > 100000) {
      work.display[4] = SEG_KILO;

      work.display[3] = a2led(buf[0]);
      work.display[2] = a2led(buf[1]);
      work.display[1] = a2led(buf[2]) | DOT_SEG;
      work.display[0] = a2led(buf[3]);
   } else if (v > 10000) {
      work.display[4] = SEG_KILO;

      work.display[3] = a2led(buf[0]);
      work.display[2] = a2led(buf[1]) | DOT_SEG;
      work.display[1] = a2led(buf[2]);
      work.display[0] = a2led(buf[3]);
   } else {
      work.display[4] = SEG_PLAIN;

      work.display[3] = a2led(buf[0]);
      work.display[2] = a2led(buf[1]);
      work.display[1] = a2led(buf[2]);
      work.display[0] = a2led(buf[3]);
   }
}


/*
 * wait for the end of frequency counting
 */
void countFrequency(void) {
#define T_BIT   0x40

   do {
      while (work.stopAt > work.timer);
      TCCR1B = 0;                                          // stop timer

      cnv2Display(work.counter.value);
      updateDisplay();

      initCounter();  
   } while (SREG & T_BIT); 
}
