/*
 * file:    Util.c
 * purpose: collect handy functions
 */
#include <inttypes.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include "Util.h"
#include "Data.h"
#include "SPI.h"
#include "SegmentLed.h"


static uint32_t PROGMEM tab[] = { 1000000000, 100000000, 10000000, 1000000,
  	                              100000, 10000, 1000, 100, 10, 1, 0 };

/*
 * Die Routine l2a() wandelt die Zahl 'l' in eine ASCII-Zeichenkette, die in
 * s[12] abgelegt wird: Vorzeichen, max. 10 Ziffern, Abschluss -> 12 Byte.
 * 
 * Zeitaufwendige Divisionen werden durch Subtraktionen ersetzt, was deutlich
 * schneller ausgef?hrt wird.
 * 
 * Bei Bedarf kann 'long' durch 'int' ersetzt werden; hierdurch wird der
 * Code reduziert, die Ausfuerung beschleunigt, aber der Wertebereich auf
 * -32768 bis 32767 begrenzt.
 * Nach Anpassung kann diese Routine auch dazu verwendet werden, die Mantisse
 * von float Zahlen sehr schnell zu wandeln.
 * 
 * Michael Nowak, 2004-11-09, www.mino-elektronik.de
 */
void l2a(int32_t l, uint8_t *s)                            // l -> zu wandelnde Zahl, *s -> Ergebnis
{
   uint32_t temp;                                          // fuer schnellere Ausfuehrung
   uint8_t  i=0;
   uint8_t  j;                                             // i -> tab-index, j -> aktuelle Ziffer

   if (l) {			                                       // wert ist != 0
      if (l < 0) {
         l = -l;
         *s++ = '-';                                       // wenn negativ
      }
      while (tab[i] > l) i++;	                           // 1. gueltige Ziffer suchen, Vornullen entfernen
      do {
         temp = tab[i];                                    // temp vermeidet wiederholte Indizierung
         j = '0';			                               // Startwert = Ziffer '0'
         if (temp) {                                       // nur wenn gueltige Tabellenwerte
    	    for (;j <= '9'; j++) {                         // mehr als '9' darf's nicht werden
    	       l -= temp;		                           // solange subtrahieren
    	       if (l < 0) {                                // bis < 0: das war einmal zu viel
    	          l += temp;                               // daher wieder korrigieren
    	          break;                                   // for-Schleife ist fertig
    	       }
    	    }
    	    *s++ = j;                                      // Ziffer ablegen
   	        i++;                                           // naechst kleinere Stelle auswerten
         }
      } while(temp);		                               // bis Tabelle abgearbeitet
   } else *s++ = '0';                                      // falls Zahl == 0, nur eine '0' erzeugen
   *s = 0;			                                       // string abschlie?en
}


/*
 * control of led-segments is handled through shift registers with output latch
 * aka 74HC595
 * Each digit has its own shift register.
 * Transfer of the 5 Bytes is done hidden and after that, the sync line is strobed
 * which causes the shift register to update their latches.
 */
void updateDisplay(void) {
   spiBusySend(work.display, sizeof(work.display));

   PORT_SYNC_COMM |= 1 << PIN_SYNC_COMM;
   __asm volatile ("nop\n");
   PORT_SYNC_COMM &= (uint8_t) ~(1 << PIN_SYNC_COMM);
}


/*
 * convert an ascii digit into the byte representation for segment-display
 */
uint8_t a2led(uint8_t digit) {
   uint8_t rv = 0;

   switch (digit) {
   case 0:  rv = SEG_0; break;
   case 1:  rv = SEG_1; break;
   case 2:  rv = SEG_2; break;
   case 3:  rv = SEG_3; break;
   case 4:  rv = SEG_4; break;
   case 5:  rv = SEG_5; break;
   case 6:  rv = SEG_6; break;
   case 7:  rv = SEG_7; break;
   case 8:  rv = SEG_8; break;
   case 9:  rv = SEG_9; break;
   }

   return rv;
}
