/*
 * file:    init.c
 * purpose: initialize device
 */
#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include "Config.h"
#include "DDS.h"
#include "Data.h"
#include "Counter.h"
#include "SPI.h"
#include "UI.h"


__attribute__((always_inline)) void listen2User(uint8_t really) {
   if (really) {
      TCCR0B = TIMER_CLOCK;                                // start timer ...
      EIMSK &= (uint8_t) ~(1 << INT1 | 1 << INT0);         // disable pin change interrupts
   } else {
      TCCR0B = 0;                                          // stop timer ...
      EIMSK |= 1 << INT1 | 1 << INT0;                      // enable pin change interrupts
   }
}


void init() {
/*
   __asm volatile ("clt");
   work.step  = 0;
   work.mode  = 0;
   work.level = 0;
   work.wave  = 0;
   work.timer = 0;

   TCCR0A     = 0;

   TCCR1A     = 0;
   TCCR1B     = 0;
 */
   DDR_RIGHT_ENCODE &= (uint8_t) ~(1 << PIN_RIGHT_ENCODE);
   DDR_LEFT_ENCODE  &= (uint8_t) ~(1 << PIN_LEFT_ENCODE);
   DDR_PUSH_ENCODE  &= (uint8_t) ~(1 << PIN_PUSH_ENCODE);

   spiInit();
   listen2User(1);
}


/*
 * setup device to work in selected mode
 */
void setup() {
   if (work.mode) {                                        // setup function generator
      listen2User(0);
      TCCR1B = 0;                                          // disable frequency counter 
      DDR_DAC = 0xFF;
   } else {                                                // setup frequency counter
      initCounter();
   } 
}


/*
 * Pin change interrupt - is activated when user interface is switched off,
 * so pressing any button or turning the wheel will reenable the user interface.
 */
ISR(wakeup) {
   listen2User(1);
}


ISR(PCINT0_vect, ISR_NAKED) {
   __asm volatile ("clt"            "\n\t"
                   "rjmp wakeup");
}

ISR(PCINT1_vect, ISR_ALIASOF(PCINT0_vect));
ISR(PCINT2_vect, ISR_ALIASOF(PCINT0_vect));


/*
 * gray-decoding by Peter Dannegger (mikrocontroller.net)
 */
ISR(TIMER0_OVF_vect) {
   static int8_t enc_last = 0x01;
   struct sm * pUI = ui + work.level;
   int8_t i=0;

    
   work.timer++;
   if (PORT_RIGHT_ENCODE & 1 << PIN_RIGHT_ENCODE) 
      i = 1;

   if (PORT_LEFT_ENCODE & 1 << PIN_LEFT_ENCODE) 
      i ^= 3;        // gray to binary


   i -= enc_last;

   if (i & 1) {
      enc_last += i;

      if (pUI->type == CHOICE_STATE || pUI->type == COUNTER_STATE) {
         *pUI->pVar += (i & 2) - 1;

         if (*pUI->pVar < 0)        *pUI->pVar = pUI->max;
         if (*pUI->pVar > pUI->max) *pUI->pVar = 0;
      }
   }

   if (PORT_PUSH_ENCODE & (1 << PIN_PUSH_ENCODE)) work.btPush++;
   else                                           work.btPush = 0;
}


/*
 * count overflow of the 16bit timer
 */
ISR(TIMER1_OVF_vect) {
   work.counter.words[1]++;
}
